@@ -13,6 +13,7 @@ import numpy as np
 from pixell import colorize, curvedsky, enmap, enplot, powspec, reproject
 
 from pspy.pspy_utils import ps_lensed_theory_to_dict
+from pspy.sph_tools import map2alm, alm2map
 
 
 class so_map:
@@ -153,6 +154,96 @@ class so_map:
             cdelt = self.data.wcs.wcs.cdelt[1]
             l_max_limit = 360 / cdelt / 4
         return l_max_limit
+        
+    def get_pixwin(self):
+        """compute the pixel window function corresponding to the map pixellisation
+        """
+        if self.pixel == "HEALPIX":
+            pixwin = hp.pixwin(self.nside)
+        if self.pixel == "CAR":
+            wy, wx = enmap.calc_window(self.data.shape)
+            pixwin = (wy[:,None] * wx[None,:])
+        return pixwin
+            
+    def convolve_with_pixwin(self, niter=3, binary=None, pixwin=None):
+        """Convolve a ``so_map`` object with a pixel window function
+        The convolution is done in harmonics space, for CAR maps
+        the pixwin is anisotropic (the pixel varies in size across the maps)
+        and the convolution is done in Fourier space.
+        We optionaly apply a binary before doing the operation to remove pathological pixels, note
+        that this operation is dangerous since we do harmonic transform of a masked map.
+        
+        Parameters
+        ----------
+        niter: integer
+          the number of iteration performed while computing the alm
+          not that for CAR niter=0 should be enough
+        binary: so_map
+            a binary mask that remove pathological pixel before doing the harmonic
+            space operation
+        pixwin: 1d array for healpix, 2d array for CAR
+            this allow you to pass a precomputed pixel window function
+        """
+
+        lmax = self.get_lmax_limit()
+            
+        if binary is not None: self.data *= binary.data
+        if pixwin is None: pixwin = self.get_pixwin()
+
+        if self.pixel == "HEALPIX":
+            alms = map2alm(self, niter, lmax)
+            alms = curvedsky.almxfl(alms, pixwin)
+            self = alm2map(alms, self)
+        if self.pixel == "CAR":
+            self = fourier_convolution(self, pixwin)
+            
+        return self
+            
+    def subtract_mean(self, mask=None):
+        """Subtract mean from a ``so_map`` object, optionnaly within a mask
+
+        Parameters
+        ----------
+        mask: either a single so_map (for ncomp = 1) or a tuple of SO map e.g (mask_T, mask_P)
+        """
+            
+        if mask is None:
+            self.data -= np.mean(self.data)
+        else:
+            if self.ncomp == 1:
+                self.data -= np.mean(self.data * mask.data)
+            else:
+                self.data[0] -= np.mean(self.data * mask[0].data)
+                self.data[1] -= np.mean(self.data * mask[1].data)
+                self.data[2] -= np.mean(self.data * mask[1].data)
+            
+        return self
+            
+    def subtract_mono_dipole(self, mask=None, bunch=24):
+        """Subtract monopole and dipole from a ``so_map`` object.
+
+        Parameters
+        ----------
+        mask: either a single so_map (for ncomp = 1) or a tuple of SO map e.g (mask_T, mask_P)
+        bunch: int
+            the bunch size (default: 24)
+        """
+
+        if self.ncomp == 1:
+            self.data = subtract_mono_dipole(
+                emap=self.data,
+                mask=None if mask is None else mask.data,
+                healpix=self.pixel == "HEALPIX",
+                bunch=bunch,
+            )
+        else:
+            for i in range(self.ncomp):
+                self.data[i] = subtract_mono_dipole(
+                    emap=self.data[i],
+                    mask=None if mask is None else mask[i if i < 2 else 1].data,
+                    healpix=self.pixel == "HEALPIX",
+                    bunch=bunch,
+                )
 
     def plot(
         self,
@@ -315,34 +406,7 @@ class so_map:
                         # enplot.show(plot,method="ipython")
                         plot.img.show()
 
-    def subtract_mono_dipole(self, mask=None, bunch=24):
-        """Subtract monopole and dipole from a ``so_map`` object.
 
-        Parameters
-        ----------
-        mask: a tuple of ``so_map`` (temperature and polarization masks)
-          a mask to put on top of the map
-        bunch: int
-          the bunch size (default: 24)
-        """
-        if mask is not None and not isinstance(mask, (list, tuple)):
-            raise ValueError("Mask must be a tuple of so_map mask!")
-
-        if self.ncomp == 1:
-            self.data = subtract_mono_dipole(
-                emap=self.data,
-                mask=None if mask is None else mask[0].data,
-                healpix=self.pixel == "HEALPIX",
-                bunch=bunch,
-            )
-        else:
-            for i in range(self.ncomp):
-                self.data[i] = subtract_mono_dipole(
-                    emap=self.data[i],
-                    mask=None if mask is None else mask[i if i < 2 else 1].data,
-                    healpix=self.pixel == "HEALPIX",
-                    bunch=bunch,
-                )
 
 
 def read_map(file, coordinate=None, fields_healpix=None, car_box=None, geometry=None):
@@ -891,3 +955,26 @@ def subtract_mono_dipole(emap, mask=None, healpix=True, bunch=24, return_values=
     if return_values:
         return map_cleaned, mono, dipole
     return map_cleaned
+
+def fourier_convolution(map_car, fourier_kernel, binary=None):
+
+    """do a convolution in fourier space with a fourier_kernel,
+    you can optionnaly use a binary to remove pathological pixels
+
+    Parameters
+    ---------
+    map_car : ``so_map`` in CAR pixellisation
+        the map to be convolved
+    fourier_kernel: 2d array
+        the convolution kernel in Fourier space
+    binary:  ``so_map``
+        a binary mask removing pathological pixels
+        
+    """
+    if binary is not None:
+        map_car.data *= binary.data
+    ft = enmap.fft(map_car.data, normalize=True)
+    ft  *= fourier_kernel
+    map_car.data = enmap.ifft(ft, normalize=True).real
+
+    return map_car
